"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_pattern_1 = __importDefault(require("url-pattern"));
// WHATWG URL API
// Supported in workers
// Supported in Node > v10.0.0
// Supported in modern browsers
// https://nodejs.org/api/url.html#url_the_whatwg_url_api
// https://caniuse.com/#feat=url
if (typeof URL === 'undefined') {
    require('universal-url').shim();
}
const patternOpts = {
    segmentNameCharset: 'a-zA-Z0-9_-',
    segmentValueCharset: 'a-zA-Z0-9@.+-_'
};
class Router {
    constructor() {
        this.routes = [];
    }
    all(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: '' });
    }
    get(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'GET' });
    }
    post(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'POST' });
    }
    put(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'PUT' });
    }
    patch(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'PATCH' });
    }
    delete(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'DELETE' });
    }
    head(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'HEAD' });
    }
    options(pattern, handler, options = {}) {
        return this.addRoute(pattern, handler, { ...options, method: 'OPTIONS' });
    }
    addRoute(pattern, handler, options = {}) {
        if (!(pattern instanceof url_pattern_1.default))
            pattern = new url_pattern_1.default(pattern, patternOpts);
        this.routes.push({ pattern, handler, options });
        return this;
    }
    findRoute(url, method) {
        if (!(url instanceof URL)) {
            url = url.startsWith('/') ? new URL(`http://domain${url}`) : new URL(url);
        }
        for (const { pattern, options, handler } of this.routes) {
            if (options.method && options.method !== method)
                continue;
            const params = pattern.match(options.matchUrl ? url.href : url.pathname);
            if (params)
                return { params, handler, url, method };
        }
        return null;
    }
    findRouteForRequest(request) {
        return this.findRoute(request.url, request.method);
    }
    handleRequest(event) {
        const route = this.findRouteForRequest(event.request);
        if (!route)
            return null;
        return route.handler({ ...route, event, request: event.request });
    }
    watch(event) {
        const handler = this.handleRequest(event);
        if (handler)
            event.respondWith(handler);
    }
    clear() {
        this.routes.length = 0;
    }
}
exports.default = Router;
//# sourceMappingURL=index.js.map